# Copyright (C) 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Volume transfer interface (v3 extension)."""

from cinderclient import base
from cinderclient.v2 import volume_transfers


class VolumeTransfer(volume_transfers.VolumeTransfer):
    pass


class VolumeTransferManager(volume_transfers.VolumeTransferManager):
    def create(self, volume_id, name=None, no_snapshots=False):
        """Creates a volume transfer.

        :param volume_id: The ID of the volume to transfer.
        :param name: The name of the transfer.
        :param no_snapshots: Transfer volumes without snapshots.
        :rtype: :class:`VolumeTransfer`
        """
        body = {'transfer': {'volume_id': volume_id,
                             'name': name}}
        if self.api_version.matches('3.55'):
            body['transfer']['no_snapshots'] = no_snapshots
            return self._create('/volume-transfers', body, 'transfer')

        return self._create('/os-volume-transfer', body, 'transfer')

    def accept(self, transfer_id, auth_key):
        """Accept a volume transfer.

        :param transfer_id: The ID of the transfer to accept.
        :param auth_key: The auth_key of the transfer.
        :rtype: :class:`VolumeTransfer`
        """
        body = {'accept': {'auth_key': auth_key}}
        if self.api_version.matches('3.55'):
            return self._create('/volume-transfers/%s/accept' % transfer_id,
                                body, 'transfer')

        return self._create('/os-volume-transfer/%s/accept' % transfer_id,
                            body, 'transfer')

    def get(self, transfer_id):
        """Show details of a volume transfer.

        :param transfer_id: The ID of the volume transfer to display.
        :rtype: :class:`VolumeTransfer`
        """
        if self.api_version.matches('3.55'):
            return self._get("/volume-transfers/%s" % transfer_id, "transfer")

        return self._get("/os-volume-transfer/%s" % transfer_id, "transfer")

    def list(self, detailed=True, search_opts=None, sort=None):
        """Get a list of all volume transfer.

        :param detailed: Get detailed object information.
        :param search_opts: Filtering options.
        :param sort: Sort information
        :rtype: list of :class:`VolumeTransfer`
        """
        resource_type = 'os-volume-transfer'
        if self.api_version.matches('3.55'):
            resource_type = 'volume-transfers'

        url = self._build_list_url(resource_type, detailed=detailed,
                                   search_opts=search_opts,
                                   sort=sort)
        return self._list(url, 'transfers')

    def delete(self, transfer_id):
        """Delete a volume transfer.

        :param transfer_id: The :class:`VolumeTransfer` to delete.
        """
        if self.api_version.matches('3.55'):
            return self._delete(
                "/volume-transfers/%s" % base.getid(transfer_id))

        return self._delete("/os-volume-transfer/%s" % base.getid(transfer_id))



# Copyright (c) 2021 Red Hat, Inc
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import importlib
import inspect

from cinderclient.tests.unit import utils
import cinderclient.v2 as cv2
import cinderclient.v3 as cv3


class v2_CompatibilityTest(utils.TestCase):

    def test_all_v2_classes_are_v3_classes(self):
        CV2 = 'cinderclient.v2'
        CV3 = 'cinderclient.v3'

        v2_class_names = []
        for mod_name, mod_data in inspect.getmembers(cv2, inspect.ismodule):
            mod = importlib.import_module(CV2 + '.' + mod_name)
            v2_class_names.extend([cls_name for cls_name, cls_data
                                   in inspect.getmembers(mod,
                                                         inspect.isclass)])

        v3_class_names = []
        for mod_name, mod_data in inspect.getmembers(cv3, inspect.ismodule):
            mod = importlib.import_module(CV3 + '.' + mod_name)
            v3_class_names.extend([cls_name for cls_name, cls_data
                                   in inspect.getmembers(mod,
                                                         inspect.isclass)])

        for cls_name in v2_class_names:
            self.assertIn(cls_name, v3_class_names)



# Copyright (C) 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Volume Backups interface (v3 extension).
"""

from cinderclient import api_versions
from cinderclient import base
from cinderclient.v2 import volume_backups


VolumeBackup = volume_backups.VolumeBackup


class VolumeBackupManager(volume_backups.VolumeBackupManager):
    @api_versions.wraps("3.9")
    def update(self, backup, **kwargs):
        """Update the name or description for a backup.

        :param backup: The :class:`Backup` to update.
        """
        if not kwargs:
            return

        body = {"backup": kwargs}

        return self._update("/backups/%s" % base.getid(backup), body)

    @api_versions.wraps("3.0")
    def create(self, volume_id, container=None,
               name=None, description=None,
               incremental=False, force=False,
               snapshot_id=None):
        """Creates a volume backup.

        :param volume_id: The ID of the volume to backup.
        :param container: The name of the backup service container.
        :param name: The name of the backup.
        :param description: The description of the backup.
        :param incremental: Incremental backup.
        :param force: If True, allows an in-use volume to be backed up.
        :param snapshot_id: The ID of the snapshot to backup. This should
                            be a snapshot of the src volume, when specified,
                            the new backup will be based on the snapshot.
        :rtype: :class:`VolumeBackup`
        """
        return self._create_backup(volume_id, container, name, description,
                                   incremental, force, snapshot_id)

    @api_versions.wraps("3.43")  # noqa: F811
    def create(self, volume_id, container=None,
               name=None, description=None,
               incremental=False, force=False,
               snapshot_id=None,
               metadata=None):
        """Creates a volume backup.

        :param volume_id: The ID of the volume to backup.
        :param container: The name of the backup service container.
        :param name: The name of the backup.
        :param description: The description of the backup.
        :param incremental: Incremental backup.
        :param force: If True, allows an in-use volume to be backed up.
        :param metadata: Key Value pairs
        :param snapshot_id: The ID of the snapshot to backup. This should
                            be a snapshot of the src volume, when specified,
                            the new backup will be based on the snapshot.
        :rtype: :class:`VolumeBackup`
        """
        # pylint: disable=function-redefined
        return self._create_backup(volume_id, container, name, description,
                                   incremental, force, snapshot_id, metadata)

    @api_versions.wraps("3.51")  # noqa: F811
    def create(self, volume_id, container=None, name=None, description=None,
               incremental=False, force=False, snapshot_id=None, metadata=None,
               availability_zone=None):
        return self._create_backup(volume_id, container, name, description,
                                   incremental, force, snapshot_id, metadata,
                                   availability_zone)

    def _create_backup(self, volume_id, container=None, name=None,
                       description=None, incremental=False, force=False,
                       snapshot_id=None, metadata=None,
                       availability_zone=None):
        """Creates a volume backup.

        :param volume_id: The ID of the volume to backup.
        :param container: The name of the backup service container.
        :param name: The name of the backup.
        :param description: The description of the backup.
        :param incremental: Incremental backup.
        :param force: If True, allows an in-use volume to be backed up.
        :param metadata: Key Value pairs
        :param snapshot_id: The ID of the snapshot to backup. This should
                            be a snapshot of the src volume, when specified,
                            the new backup will be based on the snapshot.
        :param availability_zone: The AZ where we want the backup stored.
        :rtype: :class:`VolumeBackup`
        """
        # pylint: disable=function-redefined
        body = {'backup': {'volume_id': volume_id,
                           'container': container,
                           'name': name,
                           'description': description,
                           'incremental': incremental,
                           'force': force,
                           'snapshot_id': snapshot_id, }}
        if metadata:
            body['backup']['metadata'] = metadata
        if availability_zone:
            body['backup']['availability_zone'] = availability_zone
        return self._create('/backups', body, 'backup')
